/*
 *   Copyright (c) 2005 - 2011 by Thanos Kyritsis
 *
 *   This file is part of ffpocket
 *
 *   FFpocket is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2 of the License.
 *
 *   FFpocket is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with FFpocket; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include "ffpocket.h"


void version_banner(void) {
	printf("%s Version: %s, Copyright (c) 2005-2011 Thanos Kyritsis.\n", FFPOCKET_APPNAME, FFPOCKET_VERSION); 
	printf("  built on: %s %s", __DATE__, __TIME__);
#ifdef __GNUC__
	printf(", gcc: %s\n",__VERSION__);
#else
	printf(", using a non-gcc compiler\n");
#endif

	printf("\nLinked against:\n");
#ifndef HAVE_SYSTEM_FFMPEG
	printf("  ffmpeg      %s\n", FFMPEG_VERSION);
#endif
	printf("  libavutil   %s (%d)\n", AV_STRINGIFY(LIBAVUTIL_VERSION), LIBAVUTIL_BUILD);
	printf("  libavcodec  %s (%d)\n", AV_STRINGIFY(LIBAVCODEC_VERSION), LIBAVCODEC_BUILD);
	printf("  libavformat %s (%d)\n", AV_STRINGIFY(LIBAVFORMAT_VERSION), LIBAVFORMAT_BUILD);
	printf("  libswscale  %s (%d)\n", AV_STRINGIFY(LIBSWSCALE_VERSION), LIBSWSCALE_BUILD);
#ifndef HAVE_SYSTEM_FFMPEG
	printf("  ffmpeg configuration: %s\n", FCOPTS);
#endif
}

void usage(const char *message) {
	//printf("%s Version: %s\n", FFPOCKET_APPNAME, FFPOCKET_VERSION); 
	version_banner();
	printf("\nUsage: %s [OPTIONS] -i FILE\n", FFPOCKET_APPNAME); 

	printf("\nOptions: \n");
	printf("  -i FILE                Set input filename to FILE\n");
	printf("  -o FILE                Set output filename to FILE\n");
	printf("  -c FILE                Set cache filename to FILE\n");
	printf("  --ss TIMESTAMP\t Set seek start time to timestamp\n\t\t\t (format is hh:mm:ss.xxx)\n");
	printf("  --st TIMESTAMP\t Set seek stop time to timestamp\n\t\t\t (format is hh:mm:ss.xxx)\n");
	printf("  --audio                Decode audio from input file to output file\n");
	printf("  --info                 Dump information about the given media file\n");
	printf("  --fps                  Dump fps number about the given media file\n");
	printf("  --reso                 Dump resolution number about the given media file\n");
	printf("  --delay                Dump A/V delay time number about the given media file\n");
	printf("  --duration             Dump duration/length about the given media file\n");
	printf("  -h, --help             This help information\n");
	printf("  -v, --version          Print version information\n\n");

	printf("The default action is to decode frames, not audio.\n");
	printf("If timestamp is not given, then the whole input file will be decoded to frames.\n\n");

	printf("If audio decoding is specified but not both timestamps (start/stop),\n");
	printf("then the whole stream will be decoded.\n\n");

	printf("Examples: \n");
	printf("Decode a whole avi to frames: \n");
	printf("%s -i lala.avi \n", FFPOCKET_APPNAME);
	printf("Decode only one frame, the one located right on the timestamp given:\n");
	printf("%s -i lala.avi --ss hh:mm:ss.xxx \n", FFPOCKET_APPNAME);
	printf("Decode the whole audio stream of the avi: \n");
	printf("%s -i lala.avi -o koko.wav --audio \n", FFPOCKET_APPNAME);
	printf("Decode from --ss to --st: \n");
	printf("%s -i lala.avi -o koko.wav --audio --ss hh:mm:ss.xxx --st hh:mm:ss.xxx \n", FFPOCKET_APPNAME);
	printf("\n");
	

	if (message) {
		printf("%s\n\n", message);
	}
}

void file_info(const char *input_filename) {
	int err=0;
	AVFormatContext * fcx=NULL;

	av_register_all();

	// Open the input file.
	err = av_open_input_file(&fcx, input_filename, NULL, 0, NULL);
	if(err<0){
		printf("Can't open file: %s\n", input_filename);
		exit(1);
	}

	// Find the stream info
	err = av_find_stream_info(fcx);
	
	// Give us information about the file and exit
	dump_format(fcx, 0, input_filename, FALSE);

	av_close_input_file(fcx);
}

void file_fps(const char *input_filename) {
	int err=0, i=0;
	AVFormatContext * fcx=NULL;

	av_register_all();

	// Open the input file.
	err = av_open_input_file(&fcx, input_filename, NULL, 0, NULL);
	if(err<0) {
		printf("Can't open file: %s\n", input_filename);
		exit(1);
	}

	// Find the stream info
	err = av_find_stream_info(fcx);

	// Give us information about the fps and exit
	for(i=0;i<fcx->nb_streams;i++) {
		AVStream *st = fcx->streams[i];
		if(st->codec->codec_type == CODEC_TYPE_VIDEO) {
			if(st->r_frame_rate.den && st->r_frame_rate.num)
				printf("Stream %d: %f fps(r)\n", i, av_q2d(st->r_frame_rate));
			else
				printf("Stream %d: %f fps(c)\n", i, 1/av_q2d(st->codec->time_base));
		}
	}
	
	av_close_input_file(fcx);
}

void file_reso(const char *input_filename) {
	int err=0, i=0;
	AVFormatContext * fcx=NULL;

	av_register_all();

	// Open the input file.
	err = av_open_input_file(&fcx, input_filename, NULL, 0, NULL);
	if(err<0) {
		printf("Can't open file: %s\n", input_filename);
		exit(1);
	}

	// Find the stream info
	err = av_find_stream_info(fcx);

	// Give us information about the resolution and exit
	for(i=0;i<fcx->nb_streams;i++) {
		AVStream *st = fcx->streams[i];
		if(st->codec->codec_type == CODEC_TYPE_VIDEO) {
				printf("Stream %d: %dx%d\n", i, st->codec->width, st->codec->height);
		}
	}
	
	av_close_input_file(fcx);
}

void file_delay(const char *input_filename) {
	int err=0;
	AVFormatContext * fcx=NULL;

	av_register_all();

	// Open the input file.
	err = av_open_input_file(&fcx, input_filename, NULL, 0, NULL);
	if(err<0) {
		printf("Can't open file: %s\n", input_filename);
		exit(1);
	}

	// Find the stream info
	err = av_find_stream_info(fcx);

	if (fcx->start_time != AV_NOPTS_VALUE) {
		int secs, us;
		secs = fcx->start_time / AV_TIME_BASE;
		us = fcx->start_time % AV_TIME_BASE;
		printf("Time Delay: %lld AV_TIME_BASE fractional seconds\n", fcx->start_time);
		printf("Time Delay: %d.%06d seconds\n", secs, (int)av_rescale(us, 1000000, AV_TIME_BASE));
	}

	av_close_input_file(fcx);
}

void file_duration(const char *input_filename) {
	int err=0;
	AVFormatContext * fcx=NULL;

	av_register_all();

	// Open the input file.
	err = av_open_input_file(&fcx, input_filename, NULL, 0, NULL);
	if(err<0) {
		printf("Can't open file: %s\n", input_filename);
		exit(1);
	}

	// Find the stream info
	err = av_find_stream_info(fcx);

	if (fcx->duration != AV_NOPTS_VALUE) {
		printf("Duration: %f seconds\n", ((float)fcx->duration) / AV_TIME_BASE);
		int hours, mins, secs, us;
		secs = fcx->duration / AV_TIME_BASE;
		us = fcx->duration % AV_TIME_BASE;
		mins = secs / 60;
		secs %= 60;
		hours = mins / 60;
		mins %= 60;
		printf("Duration: %02d:%02d:%02d.%01d\n", hours, mins, secs, (10 * us) / AV_TIME_BASE);
	}
	else
		printf("Duration: 0\n");

	av_close_input_file(fcx);
}

int main(int argc, char * argv[]) {
	int opt, showinfo=0, showfps=0, showreso=0, showdelay=0, showduration=0, decodeaudio=0, decodeframe=1;
	char *timepos_start=NULL, *timepos_stop=NULL, *outfile=NULL, *infile=NULL, *cachefile=NULL;
	static struct option long_options[] = {
		{"help", 0, 0, 'h'},
		{"version", 0, 0, 'v'},
		{"info", 0, 0, 'n'},
		{"ss", 1, 0, 's'},
		{"st", 1, 0, 't'},
		{"audio", 0, 0, 'a'},
		{"fps", 0, 0, 'f'},
		{"reso", 0, 0, 'r'},
		{"delay", 0, 0, 'd'},
		{"duration", 0, 0, 'e'},
		{0,0,0,0}
	};
	 
	while((opt=getopt_long(argc, argv, "c:o:i:hv", long_options, NULL))!=-1){
		switch(opt) {
			case 'i':
				infile = optarg;
				break;
			case 'o':
				outfile = optarg;
				break;
			case 's':
				timepos_start = optarg;
				break;
			case 't':
				timepos_stop = optarg;
				break;
			case 'n':
				showinfo = 1;
				decodeframe = 0;
				break;
			case 'f':
				showfps = 1;
				decodeframe = 0;
				break;
			case 'r':
				showreso = 1;
				decodeframe = 0;
				break;
			case 'd':
				showdelay = 1;
				decodeframe = 0;
				break;
			case 'e':
				showduration = 1;
				decodeframe = 0;
				break;
			case 'a':
				decodeaudio = 1;
				decodeframe = 0;
				break;
			case 'c':
				cachefile = optarg;
				break;
		   case 'v':
				version_banner();
				exit(1);
				break;
			case 'h':
			default:
				usage(NULL);
				exit(1);
				break;
		}
	}

	if (argc < 2 ) {
		usage("You must specify some options.");
		return 1;
	}

	if (showinfo) {
		if (infile)
			file_info(infile);
		else {
			usage("You must specify an input file.");
			return 1;
		}
	}

	if (showfps) {
		if (infile)
			file_fps(infile);
		else {
			usage("You must specify an input file.");
			return 1;
		}
	}

	if (showreso) {
		if (infile)
			file_reso(infile);
		else {
			usage("You must specify an input file.");
			return 1;
		}
	}

	if (showdelay) {
		if (infile)
			file_delay(infile);
		else {
			usage("You must specify an input file.");
			return 1;
		}
	}

	if (showduration) {
		if (infile)
			file_duration(infile);
		else {
			usage("You must specify an input file.");
			return 1;
		}
	}

	if (infile && outfile) {
		if (!strcmp(infile,outfile)) {
			usage("You provided the same input and output file!!");
			return 1;
		}
	}

	if (infile && cachefile) {
		if (!strcmp(infile,cachefile)) {
			usage("You provided the same input and cache file!!");
			return 1;
		}
	}

	if (decodeframe){
		if(infile && timepos_start) {
			av_register_all();
			ff_decode_frame(infile, timepos_start, TRUE);
			return 0;
		}
		else if(infile && !timepos_start) {
			av_register_all();
			ff_decode_frame(infile, timepos_start, FALSE);
			return 0;
		}
		else {
			usage("You must specify an input file and optionally a seek time position.");
			return 1;
		}
	}

	if (decodeaudio){
		if(infile && (outfile||cachefile) && !(timepos_start && timepos_stop)) {
			ff_decode_audio(outfile, infile, timepos_start, timepos_stop, FALSE, cachefile);
			return 0;
		}
		else if (infile && (outfile||cachefile)&& timepos_start && timepos_stop) {
			ff_decode_audio(outfile, infile, timepos_start, timepos_stop, TRUE, cachefile);
			return 0;
		}
		else {
			usage("You must specify BOTH an input AND an output (OR cache) file.");
			return 1;
		}
	}

	return 0;
}

